/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_i2c.h"             */
/*  Constants and definitions for I2C driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 28/02/2017       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
            2.1  I2C - "i2c_io" parameter - I2c_io
            2.2  I2C - I2C low-level - I2c_desc/I2c_drvdef/I2c_softdef
            2.3  I2C - Contexte d'un bus I2C - I2c_ctx
            2.4  EEPROM - Contexte I2C d'une EEPROM - I2c_eepctx
            2.5  EEPROM - Bas niveau EEPROM - Eep_devdesc / Eep_devdef
            2.6  EEPROM - Contexte d'une EEPROM - Eep_ctx
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef DRV_I2C_H                      /* If this file is not already        */
#define DRV_I2C_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/


/*
    2    Structures and typedefs
    ____________________________
*/


/*
   2.1  I2C - "i2c_io" parameter - I2c_io
   --------------------------------------
*/

typedef struct                         /* "i2c_io" list item                 */
  BEGIN                                /* ---------------------------------- */
    UCHAR          io                ; /* I2C address or 0xFF (wait)         */
    UCHAR          flags             ; /* Write flags                        */
    UCHAR          *adr              ; /* Transmit or receive address        */
    USHORT         lg                ; /* Count of bytes to read or write    */
  END              I2c_io            ; /* ---------------------------------- */


/*
   2.2  I2C - I2C low-level - I2c_desc/I2c_drvdef/I2c_softdef
   ----------------------------------------------------------
*/

typedef struct                         /* Low-level driver procedures addr   */
  BEGIN                                /* ---------------------------------- */
    CHAR           *name             ; /* Device name                        */
    void            (*reset)   ()    ; /* Initialization                     */
    void            (*put)     ()    ; /* Write to the bus                   */
    void            (*get)     ()    ; /* Read from the bus                  */
  END               I2c_desc         ; /* ---------------------------------- */

typedef struct                         /* Generic I2C driver parameters      */
  BEGIN                                /* ---------------------------------- */
    INT             wait             ; /* Wait duration between 2 requests   */
  END               I2c_drvdef       ; /* ---------------------------------- */

typedef struct                         /* I2C software low-level parameters  */
  BEGIN                                /* ---------------------------------- */
    UCHAR           pio_port         ; /* Numero du port PIO des fils I2C    */
    UCHAR           sda_pio          ; /* Numero du PIO du fil SDA           */
    UCHAR           scl_pio          ; /* Numero du PIO du fil SCL           */
    USHORT          hold             ; /* Temps de maintien en us            */
    USHORT          setup            ; /* Temps de chargement en us          */
  END               I2c_softdef      ; /* ---------------------------------- */


/*
   2.3  I2C - Contexte d'un bus I2C - I2c_ctx
   ------------------------------------------
*/


typedef struct                         /* Structure de controle de bcle      */
  BEGIN                                /* ---------------------------------- */
    USHORT          cnt              ; /* Compteur de boucle                 */
    UCHAR           begloop          ; /* Indice I2c_io de debut de bcle     */
  END               I2c_loop         ; /* ---------------------------------- */

#define MLOOP      3

typedef struct                         /* I2C context                        */
  BEGIN                                /* ---------------------------------- */
    void            (*reset)    ()   ; /* Controller reset    procedure      */
    void            (*put)      ()   ; /* Send one or N bytes procedure      */
    void            (*get)      ()   ; /* Bus byte read       procedure      */
                                       /*                                    */
    IWORD           sreset           ; /* RESET steps counter                */
    UINT            mode             ; /* mode, 0 if disabled                */
    UINT            speed            ; /* I2C bus speed in KHz               */
    INT             wait             ; /* Wait duration between 2 requests   */
                                       /*                                    */
    SHORT           code             ; /* Code of the REQ_.. event           */
    UCHAR           ad               ; /* I2C address or -1                  */
    UCHAR           flags            ; /* Write flags                        */
    USHORT          subad            ; /* Register selection sub-address     */
    UCHAR           *frame           ; /* Data buffer address (read/write)   */
    INT             lg               ; /* Count of bytes to be read          */
    USHORT          maxtry           ; /* Maximum tries count                */
    USHORT          rowsize          ; /* EEP row/page size in bytes         */
    SHORT           raut             ; /* Automaton to be answered to        */
    USHORT          rvl              ; /* Logical way to be answered         */
                                       /*                                    */
    UCHAR           *fracur          ; /* Read/write pointer                 */
    IWORD           try              ; /* Tries counter                      */
    IWORD           tryadr           ; /* Tries counter for the I2C address  */
    I2c_loop        loop[MLOOP]      ; /* Stack for intricated loops         */
    UCHAR           ploop            ; /* Stack pointer for "loop[]"         */
    UCHAR           ioidx            ; /* Index in the I2c_io array/list     */
                                       /*                                    */
    UINT            idto             ; /* Current running TO_COM identifier  */
    USHORT          et               ; /* "ti2c" automaton state             */
    USHORT          ev               ; /* "ti2c" automaton event             */
                                       /*                                    */
    UCHAR           msg[130]         ; /* Register slection message          */

    USHORT          block            ; /* _WORK_ temporary.                  */

  END               I2c_ctx          ; /* ---------------------------------- */


/*
   2.4  EEPROM - Contexte I2C d'une EEPROM - I2c_eepctx
   ----------------------------------------------------
*/


typedef struct                         /* Contexte I2c pour une EEPROM       */
  BEGIN                                /* ---------------------------------- */
    UCHAR          *cacheadr         ; /* Adresse du cache de l'EEPROM       */
    INT             rowsize          ; /* Taille en octets d'une rangee      */
  END               I2c_eepctx       ; /* ---------------------------------- */
 

/*
   2.5  EEPROM - Description d'un bas niveau EEPROM - Eep_devdesc / Eep_devdef
   ---------------------------------------------------------------------------
*/


typedef struct                         /* Descr. driver bas niveau EEPROM    */
  BEGIN                                /* ---------------------------------- */
    IWORD           (*init)    ()    ; /* Initialision                       */
    IWORD           (*send)    ()    ; /* Ecriture sur le bus                */
    IWORD           (*receive) ()    ; /* Lecture  sur le bus                */
    CHAR           *name             ; /* Nom du driver de bas niveau        */
  END               Eep_devdesc      ; /* ---------------------------------- */

typedef struct                            /* Parametres description EEPROM   */
  BEGIN                                   /* ------------------------------- */
    INT            bus                  ; /* Numero de bus de l'EEPROM       */
    INT            wadr                 ; /* Adresse I2C paire de l'EEPROM   */
    UCHAR          *cacheadr            ; /* Adresse RAM du cache EEPROM     */
    INT            cachesize            ; /* Taille en octets du cache RAM   */
    INT            rowsize              ; /* Taille en octets d'une rangee   */
    CHAR           *name                ; /* Nom du device EEPROM            */
  END              Eep_devdef           ; /* ------------------------------- */


/*
   2.6  EEPROM - Contexte d'une EEPROM - Eep_ctx
   ---------------------------------------------
*/

typedef struct                         /* Entree file d'attente EEP          */
  BEGIN                                /* ---------------------------------- */
    INT             code             ; /* Requete(REQ_WRITE, REQ_READ)       */
    INT             aut              ; /* Automate du demandeur.             */
    INT             vl               ; /* Voie logique du demandeur.         */
    USHORT          lg               ; /* Nombre d'octets a lire ou ecrire   */
    INT             eepadr           ; /* Adresse dans l'EEPROM              */
    INT             iod              ; /* Iod du device EEPROM concerne      */
  END               Eep_job          ; /* ---------------------------------- */

#define MAX_JOBE   10

typedef struct                         /* Contexte EEP de AUT_EEP            */
  BEGIN                                /* ---------------------------------- */
    IWORD           (*init)     ()   ; /* Reset du controlleur               */
    IWORD           (*send)     ()   ; /* Emettre 1 ou N octets              */
    IWORD           (*receive)  ()   ; /* Lire des octets sur le bus         */
                                                                             
    Eep_job         liste[MAX_JOBE]  ; /* file d'attente des travaux EEP     */
    UCHAR           lec              ; /* pointeur de lecture de la file     */
    UCHAR           ecr              ; /* pointeur d'ecriture de la file     */
    UCHAR           nbjob            ; /* Nombre d'elements de la file       */
                                                                             
    Eep_devdef     *eep              ; /* Ad structure de parametres         */
  END               Eep_ctx          ; /* ---------------------------------- */
                                                                             
typedef struct                         /* Contexte EEP procs interface       */
  BEGIN                                /* ---------------------------------- */
    UCHAR           wait_init        ; /* 1: on a recu au moins 1 reponse    */
    Eep_devdef     *eep              ; /* Ad structure de parametres         */
    Eep_devdesc    *eeplow           ; /* Ad structure driver bas niveau     */
    UCHAR           txt[STDLINE]     ; /* Buffer procedure d'ecriture        */
  END               Eepproc_ctx      ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD i2c_set             (INT,INT                                         );
  IWORD i2c_send            (INT,INT,UCHAR*,INT,INT                          );
  IWORD i2c_receive         (INT,INT,UCHAR*,INT,INT,INT                      );
  IWORD i2c_write           (INT,INT,UCHAR*,INT                              );
  IWORD i2c_read            (INT,INT,UCHAR*,INT,INT                          );
  IWORD i2c_io              (INT,I2c_io*,INT,INT                             );


#endif                                 /* End of DRV_I2C_H.                  */
