/*
**  Myapp_asy.c - Sample code for HyperpanelOS ============================  **
**                                                                           **
**  This simple code is located into the Application Container, it is run    **
**  by the VMK sub-operating system. On the other hand, the I/O container    **
**  runs all the drivers that are VMIO Finite State machines.                **
**                                                                           **
**  This application is using the driver/protocol/service API to send        **
**  requests to the ASY driver. Those requests are in fact event messages    **
**  that are received by the automatons located inside the I/O container.    **
**                                                                           **
**  The goal of this small aplication is to write a single message           **
**  "hh:mm:s <Hello World> each second to ASY0 serial port                   **
**                                                                           **
**  =======================================================================  **
*/


/* Includes files and external references ...................................*/

#include <hypos.h>                     // Hyperpanel OS basic interfaces

#include "./myio.h"                    // Interface of "myio.c"
#include "./mytsk.h"                   // INterface of "mytsk.c"


/* Internal data types of the module ........................................*/

int            loop_app_tsk     (void*                                      ) ;

static void    open_asy         (void                                       ) ;
static void    close_asy        (void                                       ) ;
static void    print_time       (char*                                      ) ;


/* Internal defines of this module ------------------------------------------*/

#define  TICK                   10000  // Code for tick event
                                
                                
/* Internal global variables of this module ---------------------------------*/

int            idto                  ; // Timer identifier
int            asy_iod0              ; // IOD of devive ASY\DEV0
unsigned char  *buf_snd0             ; // Address of DEV0 transmit buffer



/* Static data of this module -----------------------------------------------*/

static const char *asy_taglist0   = "BAUDS=115200\nNBBITS=8\n"
                                    "STOPBIT=1\nPARITY=NONE"                ;

/* Beginning of the code ----------------------------------------------------

loop_app_tsk        Entry point for create_task - Main event loop

open_asy            Open of two serial ports
close_asy           Close of two serial ports
print_time          Read time, builds a string, send it on ASY0

*/


/*  Procedure loop_app_tsk ----------------------------------------------------

    Purpose : This is our task main loop.
*/

int loop_app_tsk (void *param)
  {
    int             ev               ; // Our event

/*****************************************************************************
 * Step 1 : Here we initialize the ASY driver and we open ASY0               *
 * ------                                                                    *
 *****************************************************************************/

    start                            : // Beginning of loop label

    open_asy()                       ; // Open the two serial ports


/*****************************************************************************
 * Step 2 : We start a timer that will send us an event every 1 second so    *
 * ------   that we can update the time.                                     *
 *****************************************************************************/

    set_tto(CLOCK      ,               // Timer mode: clock
            1000       ,               // Duration in milliseconds
            TICK       ,               // Event code
            0          ,               // Event reserve field
            &idto        )           ; // Timer identifier


/*****************************************************************************
 * Step 3 : Here is our main event loop. For each loop, we do as follows :   *
 * -------  - First we wait for an event.                                    *
 *          - Then if the event code is TICK we print "Hello World"          *
 *****************************************************************************/

    wait_ev                          :

    ev = wait_allevent()             ; // Unschedule until an event is
                                       // received
    if ( ev == TICK )
      print_time("Hello World")     ;

    goto wait_ev                     ; // Wait for the next event


/*****************************************************************************
 * Step 4 : Close the ASY0 and terminates the ASY driver. In fact those      *
 * ------   instructions are never executed since loop of step 3 never ends. *
 *          However this is a didactical sample code that shows what must    *
 *          be done in order to "cleanly" terminate.                         *
 *****************************************************************************/

    clear_tto(idto)                  ; // Stop the timer
    close_asy()                      ; // Close the two serial ports

    goto start                       ;

    return 0                         ;
  }



/*  Procedure open_asy --------------------------------------------------------

    Purpose : Initialize the code of the ASY driver and then opens the serial
              port. The openend device description (Input/Output Descriptor) is
              kept in "asy_iod0". Line parameters (speed, nbbits and parity) 
              are set. Then we allocate  one transmit buffer.
*/

static void open_asy(void)
  {
    int             ret              ; // Procedures returned code

/*****************************************************************************
 * Step 1 : Open the three devices ASY\DEV0 and DEV1. We call the            *
 * -------  "myio_open" simple API procedure. In fact the subroutine calls   *
 *          "drv_init_driver", "add_uroute", "drv_alloc_device", and then    *
 *          "drv_open_device".                                               *
 *****************************************************************************/

    myio_open(DRVASY,DEV0,"",&asy_iod0) ; // Open "ASY\DEV0"


/*****************************************************************************
 * Step 2 : Set line parameters for the two serial line controllers. We have *
 * ------   to call the "myio_setval" procedure. This procedure sends a      *
 *          REQ_SETVAL request event the AUT_ASY VMIO automaton. Then we     *
 *          unschedule until the RESP_SETVAL response event is received. We  *
 *          do not use the same parameters with the UI ASY0 asynchronous     *
 *          line and the AUT_USND ASY1 line. We use asy_taglist0 with ASY0:  *
 *          "BAUDS=115200\nNBBITS=8\nSTOPBIT=1\nPARITY=NONE". With ASY1 and  *
 *          ASY2 we use taglist1: "BAUDS=9600\nNBBITS=8\nSTOPBIT=1\n"        *
 *          "PARITY=NONE\nBUFSIZE=0\nFLOWCTL=NONE\nTMODE=NONE\nTEMPO=1".     *
 *****************************************************************************/

    myio_setval(asy_iod0,asy_taglist0)  ; // Set "ASY\DEV0" tags


/*****************************************************************************
 * Step 3 : Allocates memory buffers for transmit, one for DEV0 and another  *
 * -------  one for DEV1. We store the buffers addresses in "buf_snd0" and   *
 *          "buf-snd1". It has to be noted that we do not allocate any       *
 *          receive buffer, because this is done by the ASY driver. When the *
 *          VMIO AUT_ASY receives a REQ_REQ, this event contains a number    *
 *          of allowaed receive tokens and receive buffer allocation is done *
 *          by AUT_ASY. So the present module does not contains any call     *
 *          to "alloc_buf" regarding receive buffers.                        *
 *****************************************************************************/

    alloc_buf(myapp_memuid ,           // Memory user id
              1500         ,           // Size in bytes of requested buffer
              0            ,           // Flags
              &buf_snd0    ,           // Address of allocated buffer
              &ret          )        ; // Return code

  }


/*  Procedure close_asy -------------------------------------------------------

    Purpose : This procedure closes the two USART, then frees those two 
              devices, terminates the ASY module, delete the route that has 
              been created for the IND_REPORT events, and finally frees the 2 
              telecom buffers that were allocated by "open_asy".
*/

static void close_asy(void)
  {

    int             ret              ; // Return code

/*****************************************************************************
 * Step 1 : We call the "myio_close" procedure that calls "drv_close_device",*
 * ------   "drv_free", "drv_end" and "del_uroute".                          *
 *****************************************************************************/

    myio_close(asy_iod0)             ; // Closes "ASY\DEV0"


/*****************************************************************************
 * Step 2 : Free the 2 memory buffers that were allocated by "open_asy".     *
 * ------   For each of them, we just call the "free_buffer" procedure       *
 *****************************************************************************/

    free_buf(buf_snd0, &ret )        ; // Free "buf_snd0"

  }


/*  Procedure print_time ------------------------------------------------------

    Purpose : Prints (sends) on the serial line the current time in seconds
*/

static void print_time(char * mess)
  {
    int             d, m, y          ; // Day, Month, Year
    int             h, mi, s, ms     ; // Hour, minutes, seconds, milliseconds
    int             lg               ; // Number of bytes to be sent

/*****************************************************************************
 * Step 1 : We first retrieve date and time since boot.                      *
 * ------                                                                    *
 *****************************************************************************/

    tim_get(&d  ,                      // day
            &m  ,                      // Month
            &y  ,                      // Year
            &h  ,                      // Hour (0..23)
            &mi ,                      // Minutes (0..59)
            &s  ,                      // Seconds (0..59)
            &ms   )                  ; // Milliseconds (0..999)

    hsprintf((char*)buf_snd0  ,        // Put in the transmit buffer
           "%02d:%02d:%02d <%s>\n\r" , //  HH:MM:SS
            h,   mi,  s,   mess   )  ; //
                                       //
    lg = strlen ((char*)buf_snd0   ) ; // size of the time string 


/*****************************************************************************
 * Step 2 : Send using serial DEV0, so IOD "asy_iod0", the "lg" characters   *
 * ------   that are in the buffer pointed by "buf_snd0". We call the        *
 *          "drv_write" procedure that sends a REQ_WRITE request event to    *
 *          the AUT_ASY VMIO automaton. The we unschedule until the          *
 *          RESP_WRITE response event is received                            *
 *****************************************************************************/

    myio_write(asy_iod0  ,             // I/O descriptor for ASY\DEV0
               buf_snd0  ,             // Address of buffer
               lg         )          ; // Number of bytes to be sent
  }

