/*
**  Myapp_uhf.c - Sample code for HyperpanelOS ============================= **
**                                                                           **
**  This simple code is located into the Application Container, it is run    **
**  by the VMK sub-operating system. On the other hand, the I/O container    **
**  runs all the drivers that are VMIO Finite State machines.                **
**                                                                           **
**  This code demonstrate how to use the Remote Control Unit (RCU) driver    **
**  in receive or transmit mode. When the application starts, it reads       **
**  GPIO1 and then behaves as a receiver is there is no jumper between       **
**  3V3 and GPIO1. If there is a jumper fitted between 3V3 and GPIO1 then    **
**  the applicative software behaves as a transmitter.                       **
**                                                                           **
**  In receiver mode, an UHF receiver must be installed on the UHF1          **
**  connector of the daughter board. One must use the DIO Ref 54760 UHF      **
**  Remote control unit. The softawre behaves as follows:                    **
**  - Keys 1 and 0 of line "1" will respectively switch ON and OFF the       **
**    RED LED of the daughter board                                          **
**  - Keys 1 and 0 of line "2" will respectively switch ON and OFF the       **
**    GREEN LED of the daughter board                                        **
**  - Keys 1 and 0 of line "3" will respectively switch ON and OFF the       **
**    YELLOW LED of the daughter board                                       **
**  - Keys 1 and 0 of line "G" will respectively switch ON and OFF all       **
**    the 3 RED/GREEN/YELLOW les of the daugther board                       **
**                                                                           **
**  In transmitter mode, an UHF receiver must be installed on GPIO5 of the   **
**  6 GPIO connector (1..6). The application emulates the DIO Ref 54760      **
**  Remote Control Unit as follows:                                          **
**  - A first press on BUT1 sends the Line 1 ON (1) and a second press sends **
**    the Line 1 OFF (0)                                                     **
**  - A first press on BUT2 sends the Line 2 ON (1) and a second press sends **
**    the Line 2 OFF (0)                                                     **
**  - A first press on BUT3 sends the Line 3 ON (1) and a second press sends **
**    the Line 3 OFF (0)                                                     **
**  - A first press on BUTUSR sends the  G ON (1) and a second press sends   **
**    the G OFF (0)                                                          **
**  In transmitter mode, the local LEDs blink patterns are modified in the   **
**  same way the received will do. A first press on BUT 1 will lit the RED   **
**  local led and a second press will unlit the RED led. Therfore, if one    **
**  is using a first board as a transmitter and a second board as a          **
**  receiver, the LEDS of the receiver board will "follow" the LEDs of the   **
**  transmitter board                                                        **
**  ======================================================================== **
*/

/* Includes files and external references ...................................*/

#include <hypos.h>                     // Hyperpanel OS basic interfaces
#include <drv_asy.h>                   // Prototype of "asy_write"
#include <drv_rcu.h>                   // Prototype of RCU procedures

#include "./myio.h"                    // Interface of "myio.c"
#include "./mytsk.h"                   // Interface of "mytsk.c"


/* Internal defines of this module ------------------------------------------*/

#define RECEIVER      0               // No jumper, we are a receiver
#define TRANSMITTER   1               // Jumper fitted, we are a transmitter


/* Alternate names for the DIO 54760 UHF RCU keys ...........................*/

#define KEY_1_OFF            KEY_SHIFT_F1  /* Device 1: OFF                  */
#define KEY_1_ON             KEY_SHIFT_F2  /* Device 1: ON                   */
#define KEY_2_OFF            KEY_SHIFT_F3  /* Device 2: OFF                  */
#define KEY_2_ON             KEY_SHIFT_F4  /* Device 2: ON                   */
#define KEY_3_OFF            KEY_SHIFT_F5  /* Device 3: OFF                  */
#define KEY_3_ON             KEY_SHIFT_F6  /* Device 3: ON                   */

#define KEY_GRP_OFF          KEY_SHIFT_F11 /* Group of devices: OFF          */
#define KEY_GRP_ON           KEY_SHIFT_F12 /* Group of devices: ON           */



/* Internal data types of the module ........................................*/

static void    transmitter     (void                                       ) ;
static void    receiver        (void                                       ) ;
static void    update_leds     (int                                        ) ;

static int     open_gpio       (void                                       ) ;
static void    close_gpio      (void                                       ) ;
static void    open_led        (void                                       ) ;
static void    close_led       (void                                       ) ;
static void    set_led_pattern (int, const char*                           ) ;
static void    open_kbd        (int                                        ) ;
static void    close_kbd       (void                                       ) ;


/* Internal global variables of this module ---------------------------------*/

int            uhf_mode              ; // 0:Receiver 1:Transmitter

int            gpio_iod              ; // IOD of device GPIO\DEV0
int            gpio_iodj1            ; // IOD of subchan GPIO 1 (Jumper)
int            gpio_iodb[4]          ; // IOD of subchan BUT1/BUT2/BUT3/BUTUSR

int            led_iod               ; // IOD of device LED\DEV0
int            led_iods[3]           ; // IOD of subchan RED/GREEN/YELLOW

int            kbd_iod               ; // IOD of device KBDITF\DEV0
int            rcusnd_id             ; // ID for RCU SND
int            rcurcv_id             ; // ID for RCU RCV
unsigned char  buf_rcu[160]          ; // RCU send buffer


/* Static data of this module -----------------------------------------------*/

static const char *const gpio_tag_jump = 
               "FUNC=GPIO\nOUTPUT=HI_Z\nWEAKPULL=DOWN\nEVENTS=OFF"           ;

static const char *const gpio_tag_but = 
               "FUNC=GPIO\nOUTPUT=HI_Z\nWEAKPULL=UP\nEVENTS=REPORT"          ;

static const char *const led_tag_pop  = 
               "CMD=POPALL"                                                  ;

static const char *const led_tag_off =    
               "PATTERN=OFF:0\nCMD=PUSH"                                     ;

static const char *const led_tag_on =           
               "PATTERN=ON:0\nCMD=PUSH"                                      ;


/* Beginning of the code ----------------------------------------------------

loop_app_tsk        Main loop of the Application container task
transmitter         Transmitter mode handling
receiver            Receiver mode handling
update_leds         Update LEDs according to the received KEY

------------------  User task devices initialization and termination --------
open_gpio           Open the GPIO driver
close_gpio          Close the GPIO driver
open_led            Open the LED driver
close_led           Close the LED driver
set_led_pattern     Set a blinking pattern to one LED
open_kbd            Open the KBDITF driver
close_kbd           Close the KBDITF driver

*/

/*  Procedure loop_app_tsk ----------------------------------------------------

    Purpose : This is our task main loop.
*/

int loop_app_tsk (void *param)
  {

/*****************************************************************************
 * Step 1 :
 *****************************************************************************/

    uhf_mode = open_gpio()           ; // Open the GPIO driver and sample
                                       // GPIO 1 state

    open_led()                       ; // Open the LED driver
    open_kbd(uhf_mode)               ; // Open the KBDITF driver


/*****************************************************************************
 * Step 2 :
 *****************************************************************************/

    if ( uhf_mode )
      transmitter()                  ;
    else
      receiver()                     ;


/*****************************************************************************
 * Step 3 :
 *****************************************************************************/

    close_gpio()                     ; // Close the GPIO driver
    close_led()                      ; // Close the LED driver
    close_kbd()                      ; // Close the KBDITF driver

    return 0                         ;
  }


/*  Procedure transmitter -----------------------------------------------------

    Purpose : 
*/

static void transmitter(void)
  {
  }


/*  Procedure receiver --------------------------------------------------------

    Purpose : Main loop of the "receiver" function. The loop receives from
              "drv_rcu" DRV_KEY_PRESS and DRV_KEY_RELEASE events. With
              DRV_KEY_PRESS events, we change the local LED state according
              to the received keycode. The DRV_KEY_RELEASE events are not
              used.
*/

static void receiver(void)
  {
    int             evtcode          ; // Received event code
    int             keycode          ; // Received RCU keycode (evt reserve)
    char            msg[80]          ; // Trace message

/*****************************************************************************
 * Step 1 :
 *****************************************************************************/

    loop                             :

    wait_allevent()                  ;
    evtcode = task_evt.code          ;
    keycode = task_evt.reserve       ;

    hsprintf(msg                   ,
     "evtcode %d  keycode 0x%08X\r\n",
       evtcode                     ,
       keycode                      );

    asy_write(0            ,
       (unsigned char*)msg ,
       strlen(msg)          )        ;

    if ( evtcode EQ DRV_KEY_PRESS )
      update_leds ( keycode )        ;

    goto loop                        ;
  }


/*  Procedure update_leds -----------------------------------------------------

    Purpose : Update the led according to a DIO keycode. The DIO Ref 54760
              UHF RCU has 8 buttons lebelled I1/O1, I2/O2, I3/O3 and IG/OG.
              According to the keycode, what we do with the 3 LEDs is as
              follows:

              I1 -> RED ON    O1 -> RED OFF
              I2 -> RED ON    O2 -> RED OFF
              I3 -> RED ON    O3 -> RED OFF
              IG -> ALL ON    OG -> ALL OFF
*/

static void update_leds(int keycode)
  {
    switch ( keycode )
      {
        case KEY_1_OFF   :                 // Key is O1
          set_led_pattern(0,led_tag_off) ; // Set RED OFF
          break                          ; //

        case KEY_1_ON    :                 // Key is I1
          set_led_pattern(0,led_tag_on ) ; // Set RED ON
          break                          ; //

        case KEY_2_OFF   :                 // Key is O2
          set_led_pattern(1,led_tag_off) ; // Set GREEN OFF
          break                          ; //

        case KEY_2_ON    :                 // Key is I2
          set_led_pattern(1,led_tag_on ) ; // Set GREEN ON
          break                          ; //

        case KEY_3_OFF   :                 // Key is O3
          set_led_pattern(2,led_tag_off) ; // Set YELLOW OFF
          break                          ; //

        case KEY_3_ON    :                 // Key is I3
          set_led_pattern(2,led_tag_on ) ; // Set YELLOW ON
          break                          ; //

        case KEY_GRP_OFF :                 // Key is OG
          set_led_pattern(0,led_tag_off) ; // Set RED OFF
          set_led_pattern(1,led_tag_off) ; // Set GREEN OFF
          set_led_pattern(2,led_tag_off) ; // Set YELLOW OFF
          break                          ;

        case KEY_GRP_ON  :                 // Key is IG
          set_led_pattern(0,led_tag_on ) ; // Set RED ON
          set_led_pattern(1,led_tag_on ) ; // Set GREEN ON
          set_led_pattern(2,led_tag_on ) ; // Set YELLOW ON
          break                          ;
      }

  }


/*  Procedure open_gpio -------------------------------------------------------

    Purpose : Initialize the code of the GPIO driver module, then allocates
              the GPIO controller device, initializes its hardware and finally
              creates a user route in order to catch the IND_REPORT events.
              The input/output descriptor is kept in the global variable 
              "gpio_iod" for the GPIO controller and the sub-channels
              descriptors are "gpio_iodj1" for the GPIO 1 (Jumper) and
              "gpio_iodb[4]" for the BUT1/BUR2/BUT2 (only in transmitter mode)
*/

static int open_gpio(void)
  {

    char            tagstr[80]       ; // Tag name then Tag+value
    int             val              ; // 0:LOW 1:HIGH


/*****************************************************************************
 * Step 1 : We call the "myio_open" that calls "drv_init_driver", then       *
 * ------   "add_uroute", then "drv_alloc_device" and "drv_open_device".     *
 *          The "gpio_iod" output parameter is the IOD for the GPIO\DEV0     *
 *          controller.                                                      *
 *****************************************************************************/

    myio_open(DRVGPIO,DEV0,"",&gpio_iod); // Open GPIO\DEV0


/*****************************************************************************
 * Step 2 : We sample the GPIO1 in order to detect if we have a jumper       *
 * ------   fitted or not. If the jumper is fitted, it establishes a         *
 *          short circuit between 3V3 and GPIO1, thus GPIO1 state is HIGH.   *
 *          Indeed we would like the GPIO1 state to be LOW if the jumper is  *
 *          not fitted and so we have to configure the GPIO with output      *
 *          drivers disabled (OUTPUT=HI_Z) and a weak pull-down resistor     *
 *          (WEAKPULL=DOWN). What we have to do here is as follows:          *
 *          - We allocated GPIO1, using its username UNAME=GPIO1. We call    *
 *            "myio_alloc_sub" that returns on output the "gpio_iodj1"       *
 *            subchannel IOD.                                                *
 *          - We configure GPIO1 in order to detect if we have or not a      *
 *            jumper fitted between 3V3 and GPIO1, so output drivers are     *
 *            disabled and a weak pull-down resistor is connected. We call   *
 *            "myio_setval" with the "gpio_iod1j" subchannel and the list of *
 *            tags "FUNC=GPIO\nOUTPUT=HI_Z\nWEAKPULL=DOWN\nEVENTS=OFF"       *
 *          - Now the GPIO1 is properly configured we sample it once, using  *
 *            a call to "myio_getval" to read the INPUT tag current value:   *
 *            . We have to set a variable ("tagstr") with the name go the    *
 *              tags to be read, here only the INPUT tag name.               *
 *            . We call "myio_setval" that sends a REQ_GETVAL to the         *
 *              AUT_GPIO IO container FSM and then unschedules us until the  *
 *              RESP_GETVAL is received.                                     *
 *            . The output string value is "INPUT=LOW" or "INPUT=HIGH". So   *
 *              if strcmp(tagstr,"INPUT=HIGH") is 0 the strings are equal    *
 *              and we set "val" to 1 (signal is HIGH, jumper is set)        *
 *          If the jumper is set, if "val" is 1, then we are a transmitter   *
 *          and we will use the 4 buttons. If we are a receiver, if "val" is *
 *          0, we are a receiver and the buttons will not be used, so we     *
 *          skip step 2, we are done.                                        *
 *****************************************************************************/

    myio_alloc_sub(gpio_iod      ,     // We allocate the UNAME=GPIO1
                   "UNAME=GPIO1" ,     // GPIO signal and get its subchannel
                   &gpio_iodj1    )  ; // IOD in "gpio_iodj1"

    myio_setval(gpio_iodj1,            // Disable the output drivers and
                gpio_tag_jump     )  ; // enable a weak pull-down resistor

    strcpy(tagstr,"INPUT")           ; // Name of the tag to be read

    myio_getval(gpio_iodj1   ,         // Sub-channel IOD
                tagstr       ,         // Input and output string
                sizeof(tagstr))      ; // String maximum size

    val = strcmp(tagstr,               // strcmp returns 0 is strings are
                 "INPUT=HIGH") ?       // equal and a !0 value else
                 RECEIVER      :       // tagstr <> "INPUT=HIGH -> RECEIVER
                 TRANSMITTER         ; // tagstr == "INPUT=HIGH -> TRANSMITTER

    if ( val EQ 0 )                    // If we are a receiver, we do not use
      goto end                       ; // the buttons and we skip step 2


/*****************************************************************************
 * Step 2 : We are a transmitter. We use the 4 push-buttons, that are NO     *
 * ------   (Normally Open) grounded switches. We use a weak pull-up         *
 *          resistor (WEAKPULL=UP) and so when the button is not pressed     *
 *          the switch is open and the GPIO is 3V3 (HIGH) and if the button  *
 *          is pressed the switch is closed and the GPIO is 0V (LOW). What   *
 *          we have to do here is:                                           *
 *          - Allocated the 4 GPIO that are connected to the 4 buttons, one  *
 *            for each button. The usernames are respectively UNAME=BUT1,    *
 *            UNAME=BUT2, UNAME=BUT3 and UNAME=BUTUSR. We call 4 times the   *
 *            "drv_alloc_subchan" procedure, this procedure sends a          *
 *            REQ_ALLOC_SUB event to the VMIO AUT_GPIO automaton. This       *
 *            automaton then send us a RESP_ALLOC_SUB response event. Here   *
 *            each of the 5 calls is unschedule until the corresponding      *
 *            RESP_ALLOC8SUB event is received.                              *
 *          - The 4 pins are allocated. We need to configure the GPIOs to    *
 *            get IND_REPORT events on state change. We call the             *
 *            "myio_setval" procedure that sends a REQ_SETVAL event to the   *
 *            VMIO AUT_GPIO automaton. This automaton will then send us a    *
 *            RESP_SETVAL response event. Here "myio_setval" unschedules     *
 *            until this event is received. We do this for buttons 1, 2, 3   *
 *            and USR, so using the four sub-channels IOD, "gpio_iodb[0] to  *
 *            "gpio_iodb[3]". We use the same tag list for the 4 GPIO's that *
 *            are connected to the "Normally Open" push buttons. Therefore   *
 *            we disable the output drivers and set a weak pull-up resistor. *
 *            We call "myio_setval" with the "gpio_iodb[]" sub-channel IOD   *
 *            and "FUNC=GPIO\nOUTPUT=HI_Z\nWEAKPULL=UP\nEVENTS=REPORT\n".    *
 *****************************************************************************/

    myio_alloc_sub(gpio_iod,"UNAME=BUT1"  ,&gpio_iodb[0]); // Alloc BUT0
    myio_alloc_sub(gpio_iod,"UNAME=BUT2"  ,&gpio_iodb[1]); // Alloc BUT1
    myio_alloc_sub(gpio_iod,"UNAME=BUT3"  ,&gpio_iodb[2]); // Alloc BUT2
    myio_alloc_sub(gpio_iod,"UNAME=BUTUSR",&gpio_iodb[3]); // Alloc BUTUSR

    myio_setval(gpio_iodb[0],gpio_tag_but) ; // Configure BUTO   GPIO
    myio_setval(gpio_iodb[1],gpio_tag_but) ; // Configure BUT1   GPIO
    myio_setval(gpio_iodb[2],gpio_tag_but) ; // Configure BUT2   GPIO
    myio_setval(gpio_iodb[3],gpio_tag_but) ; // Configure BUTUSR GPIO

    end : 
    return val ;
 }


/*  Procedure close_gpio ------------------------------------------------------

    Purpose : This procedure closes the GPIO controller, then frees this
              device, terminates the GPIO module and deletes the route that
              had been created for the IND_REPORT events.
*/

static void close_gpio(void)
  {

/*****************************************************************************
 * Step 1 : Free all used subchannels. We call "myio_free_sub" that calls    *
 * ------   "drv_free_subchan" for each GPIO to be freed:                    *
 *          - Whatever we have a receiver or a transmitter, we free GPIO1    *
 *            (jumper), so "gpio_iodj1"                                      *
 *          - If we are a transmitter, we have also to free BUT1/BUT2/BUT2   *
 *            and BUTUSR, so "gpio_iodb[0]" to "gpio_iodb[3]". In fact, as   *
 *            an IOD canot be a 0 value, we just test "gpio_iodb[0]" and if  *
 *            it is 0, the GPIO buttons have not been allocated.             *
 *          The "myio_free_sub" procedure calls "drv_free_subchan" that      *
 *          sends a REQ_FREE_SUB request event to the VMIO AUT_GPIO          *
 *          automaton and here we unschedule until the corresponding         *
 *          RESP_FREE_SUB, sent by AUT_GPIO, is received.                    *
 *****************************************************************************/

    myio_free_sub(gpio_iodj1  )      ; // Free GPIO1

    if (gpio_iodb[0] EQ 0)             // If BUT1/BUT2/BUT3 and BUTUSR have
      goto step2                     ; // not been allocated, goto step2

    myio_free_sub(gpio_iodb[0])      ; // Free BUT1
    myio_free_sub(gpio_iodb[1])      ; // Free BUT2
    myio_free_sub(gpio_iodb[2])      ; // Free BUT3
    myio_free_sub(gpio_iodb[3])      ; // Free BUTUSR


/*****************************************************************************
 * Step 2 : Close the GPIO device. We call the "myio_close" subroutine that  *
 * ------   calls "drv_close" device, the "drv_free" and then "drv_end".     *
 *          A REQ_CLOSE request event is sent to AUT_GPIO and we unschedule  *
 *          until RESP_CLOSE is received from AUT_GPIO. A REQ_FREE event is  *
 *          sent and RESP_FREE is awaited. And a REQ_END driver termination  *
 *          request is sent and we unschedule until RESP_END is received.    *
 *****************************************************************************/

    step2                            : // Closes the GPIO device
    myio_close(gpio_iod)             ; // and terminates the driver
  }


/*  Procedure open_led --------------------------------------------------------

    Purpose : Initializes the LED driver, opens the LED controller, allocates
              the three LED (RED, GREEN, YELLOW) subchannels.
*/

static void open_led(void)
  {

/*****************************************************************************
 * Step 1 : We call the "myio_open" that calls "drv_init_driver", then       *
 * ------   "add_uroute", then "drv_alloc_device" and "drv_open_device".     *
 *          It has to be noted that the LED driver is a "software device",   *
 *          because there is no LED controller dedicated device. Most of the *
 *          time "hidden" GPIO (not visible by the application) are used.    *
 *****************************************************************************/

    myio_open(DRVLED,DEV0,"",&led_iod)  ; // Open LED\DEV0


/*****************************************************************************
 * Step 2 : The contoller is ready. We are going to allocate the LED         *
 * ------   subchannels, one for each LED. We call the "myio_alloc_sub"      *
 *          subroutine that calls the "drv_alloc_subchan" procedure. This    *
 *          last procedure sends a REQ_ALLOC_SUB event to the VMIO AUT_LED   *
 *          automaton. This automaton will then send us a RESP_ALLOC_SUB     *
 *          response event. Here we unschedule until this event is received. *
 *****************************************************************************/

    myio_alloc_sub(led_iod,"LEDNAME=RED"    ,&led_iods[0]); // Allocates RED
    myio_alloc_sub(led_iod,"LEDNAME=GREEN"  ,&led_iods[1]); // Allocates GREEN
    myio_alloc_sub(led_iod,"LEDNAME=YELLOW" ,&led_iods[2]); // Allocates YELLOW
  }


/*  Procedure close_led -------------------------------------------------------

    Purpose : Closes the LED controller, then frees it and terminates the LED
              driver module.
*/

static void close_led(void)
  {

/*****************************************************************************
 * Step 1 : Free all used subchannels. We call "myio_free_sub" that calls    *
 * -------  "drv_free_subchan" for each of the 3 LEDs which IO descriptor    *
 *          are "led_iodr/g/y". This last procedure sends a REQ_FREE_SUB     *
 *          request event to the VMIO AUT_LED automaton and here we          *
 *          unschedule until the corresponding RESP_FREE_SUB, sent by        *
 *          AUT_LED, is received.                                            *
 *****************************************************************************/

    myio_free_sub(led_iods[0])       ; // Frees RED
    myio_free_sub(led_iods[1])       ; // Frees GREEN
    myio_free_sub(led_iods[2])       ; // Frees YELLOW


/*****************************************************************************
 * Step 2 : We call the "myio_close" procedure that calls "drv_close_device",*
 * ------   "drv_free", "drv_end" and "del_uroute".                          *
 *****************************************************************************/

    myio_close(led_iod)              ; // Closes LED device and driver
  }


/*  Procedure set_led_pattern -------------------------------------------------

    Purpose : Sets a new blinking parten for LED "n" (0:RED, 1:GREEN, 2:YELLOW)
*/

static void set_led_pattern(int n, const char *pat)
  {
    myio_setval(led_iods[n], led_tag_pop) ;
    myio_setval(led_iods[n], pat        ) ;
  }


/*  Procedure open_kbd --------------------------------------------------------

    Purpose : Initialize the code of the KBDITF driver and then open the
              device to get key press events from the remote control unit 
              (RCU). The opened device descriptor (Input/Output Descriptor) is
              kept in "kbd_iod".
*/

static void open_kbd(int mode)
  {

    int             ret              ; // Procedures returned code

/*****************************************************************************
 * Step 1 : Open the "KBDITF\DEV0" device. We call the "myio_open" simple    *
 * ------   API procedure. In fact the subroutine calls "drv_init_driver",   *
 *          "add_uroute", "drv_alloc_device", and then the "drvopen_device". *
 *          The "KBDITF\DEV0" driver is a software driver that does not      *
 *          handles any hardware by itself. With this sample application,    *
 *          the underlying hardware is an infra-red receiver. This hardware  *
 *          is managed by the "drv_rcu" driver, that in our case sends       *
 *          events to the KBDITF module, that sends DRV_KEY_PRESS and        *
 *          DRV_KEY_RELEASE events to us.                                    *
 *****************************************************************************/

    myio_open(DRVKBD,DEV0,"",&kbd_iod) ; // Opens KBDITF\DEV0

    if ( mode == TRANSMITTER )
      goto step3                       ;


/*****************************************************************************
 * Step 2 : We are a receiver.
 *****************************************************************************/

    rcu_get_id(0          ,              // 0:Receiver 1:Transmitter
               0          ,              // numdev = the first one
               &rcurcv_id ,              // Corresponding id
               &ret        )           ; // Error code

    rcu_set(rcurcv_id   ,                // id
            1           ,                // swpos = the selected input (IR/UHF)
            "DIO 54760" ,                // name  =
            0xFFFFFFFF  ,                // idval =
            &ret          )            ; // ret   = error code

    goto end                           ;

/*****************************************************************************
 * Step 3 : We are a transmitter
 *****************************************************************************/
    step3                              :

    rcu_get_id(1          ,              // 0:Receiver 1:Transmitter
               0          ,              // numdev = the first one
               &rcusnd_id ,              // Corresponding id
               &ret        )           ; // Error code

    end :                              ;
  }


/*  Procedure close_kbd -------------------------------------------------------

    Purpose : This procedure closes the keyboard interface, frees the device,
              terminates the KBDITF module, deletes the route that has been 
              created for the DRV_KEY_PRESS and DRV_KEY_RELEASE events.
*/

static void close_kbd(void)
  {

/*****************************************************************************
 * Step 1 : We call the "myio_close" procedure that calls "drv_close_device",*
 * ------   "drv_free", "drv_end" and "del_uroute". A REQ_CLOSE is sent to   *
 *          AUT_KBD and our task is unscheduled until we receive from        *
 *          the AUT_KBD FSM the RESP_CLOSE response event. Then we send a    *
 *          REQ_FREE event request and we unschedule until RESP_FREE is      *
 *          received from AUT_KBD. And we send a REQ_END and unschedule,     *
 *          waiting for the RESP_EVENT response event.                       *
 *****************************************************************************/

    myio_close(kbd_iod)              ; // Closes KBDITF\DEV0

  }







